// RedHat6.2 iostream complains with -Dbool=char
#undef bool
#include <iostream.h>
#include "util/PlatformUtils.hpp"
#include "PerlExceptionHandler.hpp"
#include "util/XMLString.hpp"
#include "util/XMLException.hpp"
#include "dom/DOM_DOMException.hpp"
#include "idom/IDOM_DOMException.hpp"
#include "sax/SAXException.hpp"

// initialize the static instance pointer
PerlExceptionHandler* PerlExceptionHandler::instance = NULL;

PerlExceptionHandler::PerlExceptionHandler() {
	// we initialize the exceptionHandler
    XMLExceptionHandler = NULL;
    IDOMExceptionHandler = NULL;
    DOMExceptionHandler = NULL;
    SAXNotRecognizedExceptionHandler = NULL;
    SAXNotSupportedExceptionHandler = NULL;
}

PerlExceptionHandler::~PerlExceptionHandler() {
    if (XMLExceptionHandler != NULL) {
	SvREFCNT_dec(XMLExceptionHandler);
	XMLExceptionHandler = NULL;
    }
    if (IDOMExceptionHandler != NULL) {
	SvREFCNT_dec(IDOMExceptionHandler);
	IDOMExceptionHandler = NULL;	
    }
    if (DOMExceptionHandler != NULL) {
	SvREFCNT_dec(DOMExceptionHandler);
	DOMExceptionHandler = NULL;	
    }
    if (SAXNotRecognizedExceptionHandler != NULL) {
	SvREFCNT_dec(SAXNotRecognizedExceptionHandler);
	SAXNotRecognizedExceptionHandler = NULL;	
    }
    if (SAXNotSupportedExceptionHandler != NULL) {
	SvREFCNT_dec(SAXNotSupportedExceptionHandler);
	SAXNotSupportedExceptionHandler = NULL;	
    }
}

PerlExceptionHandler*
PerlExceptionHandler::getInstance() {
    if (instance == NULL) {
	instance = new PerlExceptionHandler();
    }
    return instance;
}

SV *
PerlExceptionHandler::setXMLExceptionHandler(SV *codeRef) {
    SV *oldRef = &PL_sv_undef;	// default to 'undef'
    if (SvTYPE( SvRV(codeRef)) == SVt_PVCV) {
	if (XMLExceptionHandler != NULL) {
	    oldRef = XMLExceptionHandler;
	    SvREFCNT_dec(oldRef);
	}
        SvREFCNT_inc(codeRef);
        XMLExceptionHandler = codeRef;
    } else {
        croak("\nsetXMLExceptionHandler: invalid argument, must be code reference.\n");
    }
    return oldRef;
}

SV *
PerlExceptionHandler::setDOMExceptionHandler(SV *codeRef) {
    SV *oldRef = &PL_sv_undef;	// default to 'undef'
    if (SvTYPE( SvRV(codeRef)) == SVt_PVCV) {
	if (DOMExceptionHandler != NULL) {
	    oldRef = DOMExceptionHandler;
	    SvREFCNT_dec(oldRef);
	}
        SvREFCNT_inc(codeRef);
        DOMExceptionHandler = codeRef;
    } else {
        croak("\nsetDOMExceptionHandler: invalid argument, must be code reference.\n");
    }
    return oldRef;
}

SV *
PerlExceptionHandler::setIDOMExceptionHandler(SV *codeRef) {
    SV *oldRef = &PL_sv_undef;	// default to 'undef'
    if (SvTYPE( SvRV(codeRef)) == SVt_PVCV) {
	if (IDOMExceptionHandler != NULL) {
	    oldRef = IDOMExceptionHandler;
	    SvREFCNT_dec(oldRef);
	}
        SvREFCNT_inc(codeRef);
        IDOMExceptionHandler = codeRef;
    } else {
        croak("\nsetIDOMExceptionHandler: invalid argument, must be code reference.\n");
    }
    return oldRef;
}

SV *
PerlExceptionHandler::setSAXNotRecognizedExceptionHandler(SV *codeRef) {
    SV *oldRef = &PL_sv_undef;	// default to 'undef'
    if (SvTYPE( SvRV(codeRef)) == SVt_PVCV) {
	if (SAXNotRecognizedExceptionHandler != NULL) {
	    oldRef = SAXNotRecognizedExceptionHandler;
	    SvREFCNT_dec(oldRef);
	}
        SvREFCNT_inc(codeRef);
        SAXNotRecognizedExceptionHandler = codeRef;
    } else {
        croak("\nsetSAXNotRecognizedExceptionHandler: invalid argument, must be code reference.\n");
    }
    return oldRef;
}

SV *
PerlExceptionHandler::setSAXNotSupportedExceptionHandler(SV *codeRef) {
    SV *oldRef = &PL_sv_undef;	// default to 'undef'
    if (SvTYPE( SvRV(codeRef)) == SVt_PVCV) {
	if (SAXNotSupportedExceptionHandler != NULL) {
	    oldRef = SAXNotSupportedExceptionHandler;
	    SvREFCNT_dec(oldRef);
	}
        SvREFCNT_inc(codeRef);
        SAXNotSupportedExceptionHandler = codeRef;
    } else {
        croak("\nsetSAXNotSupportedExceptionHandler: invalid argument, must be code reference.\n");
    }
    return oldRef;
}

void 
PerlExceptionHandler::catchXMLException(const XMLException &e) {
    if (!XMLExceptionHandler) {
	char *errMsg = XMLString::transcode(e.getMessage());
	XMLPlatformUtils::Terminate();
	croak("\nReceived XMLException without handler\nException thrown in file: '%s'\nAt line: %s\nException message is: \n%s\n"
	     , e.getSrcFile() 
	     , e.getSrcLine() 
	     , errMsg);
	delete [] errMsg;
    }
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);

        // first argument is the element name
    char *class_name = "XML::Xerces::XMLException";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)&e), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));

    PUTBACK;

    perl_call_sv(XMLExceptionHandler, G_VOID);

    FREETMPS;
    LEAVE;
}

void 
PerlExceptionHandler::catchDOMException(const DOM_DOMException &e) {
    if (!DOMExceptionHandler) {
	char *errMsg = e.msg.transcode();
	XMLPlatformUtils::Terminate();
	croak("\nReceived DOMException without handler\nException code is: '%s'\nException message is: \n%s\n",
	      e.code,
	      errMsg);
	delete [] errMsg;
    }
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);

        // first argument is the element name
    char *class_name = "XML::Xerces::DOM_DOMException";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)&e), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));

    PUTBACK;

    perl_call_sv(DOMExceptionHandler, G_VOID);

    FREETMPS;
    LEAVE;
}

void 
PerlExceptionHandler::catchIDOMException(const IDOM_DOMException &e) {
    if (!IDOMExceptionHandler) {
	char *errMsg = XMLString::transcode(e.msg);
	XMLPlatformUtils::Terminate();
	croak("\nReceived IDOMException without handler\nException code is: %s\nException message is: %s\n",
	      e.code,
	      errMsg);
	delete [] errMsg;
    }
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);

        // first argument is the element name
    char *class_name = "XML::Xerces::IDOM_DOMException";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)&e), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));

    PUTBACK;

    perl_call_sv(IDOMExceptionHandler, G_VOID);

    FREETMPS;
    LEAVE;
}

void 
PerlExceptionHandler::catchSAXNotRecognizedException(const SAXNotRecognizedException &e) {
    if (!SAXNotRecognizedExceptionHandler) {
	char *errMsg = XMLString::transcode(e.getMessage());
	XMLPlatformUtils::Terminate();
	croak("\nReceived SAXNotRecognizedException without handler\nException message is: %s\n",
	      errMsg);
	delete [] errMsg;
    }
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);

        // first argument is the element name
    char *class_name = "XML::Xerces::SAXNotRecognizedException";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)&e), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));

    PUTBACK;

    perl_call_sv(SAXNotRecognizedExceptionHandler, G_VOID);

    FREETMPS;
    LEAVE;
}

void 
PerlExceptionHandler::catchSAXNotSupportedException(const SAXNotSupportedException &e) {
    if (!SAXNotSupportedExceptionHandler) {
	char *errMsg = XMLString::transcode(e.getMessage());
	XMLPlatformUtils::Terminate();
	croak("\nReceived SAXNotSupportedException without handler\nException message is: %s\n",
	      errMsg);
	delete [] errMsg;
    }
    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);

        // first argument is the element name
    char *class_name = "XML::Xerces::SAXNotSupportedException";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)&e), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));

    PUTBACK;

    perl_call_sv(SAXNotSupportedExceptionHandler, G_VOID);

    FREETMPS;
    LEAVE;
}
