#!/usr/bin/perl
use lib '.';
use SWIG qw(remove_method skip_to_closing_brace fix_method);

use strict;

my $file = shift @ARGV;
my $PRINTED = 0;
my $temp_file = "$file.$$";

open(FILE, $file)
  or die "Couldn't open $file for reading";
open(TEMP, ">$temp_file")
  or die "Couldn't open $temp_file for writing";

#
# Set up the regexp for matching methods that need to hand DOM_DOMException's
#
my @dom_methods = qw(IDOMParser::parse
		     IDOM_CharacterData::appendData
		     IDOM_CharacterData::deleteData
		     IDOM_CharacterData::getData
		     IDOM_CharacterData::insertData
		     IDOM_CharacterData::replaceData
		     IDOM_CharacterData::substringData
		     IDOM_DOMImplementation::createDocument 
		     IDOM_DOMImplementation::createDocumentType
		     IDOM_Element::removeAttribute 
		     IDOM_Element::removeAttributeNS 
		     IDOM_Element::removeAttributeNode 
		     IDOM_Element::setAttribute 
		     IDOM_Element::setAttributeNS 
		     IDOM_Element::setAttributeNode 
		     IDOM_Element::setAttributeNodeNS 
		     IDOM_NamedNodeMap::removeNamedItem 
		     IDOM_NamedNodeMap::removeNamedItemNS 
		     IDOM_NamedNodeMap::setNamedItem 
		     IDOM_NamedNodeMap::setNamedItemNS 
		     IDOM_Node::appendChild 
		     IDOM_Node::getNodeValue 
		     IDOM_Node::insertBefore 
		     IDOM_Node::removeChild 
		     IDOM_Node::replaceChild 
		     IDOM_Node::setPrefix 
		     IDOM_NodeIterator::nextNode 
		     IDOM_NodeIterator::previousNode 
		     IDOM_ProcessingInstruction::getData 
		     IDOM_Text::splitText 
		     IDTDHandler::notationDecl 
		     IDTDHandler::unparsedEntityDecl 
		     IDOM_DOMImplementation::createDocument
		    );
my $dom_methods = join '|', 
  map {s/^DOM/I?DOM/; s/::/_/; "_wrap_$_(?:__overload__\\d)?"} @dom_methods;

# now for the methods that only raise XMLExceptions
my @xml_methods = qw(
		     Attributes::getURI
		    );
my $xml_methods = join '|', 
  map {s/::/_/; "_wrap_$_"} @xml_methods;

my $xmlexception_regexp = qr/XS\(($xml_methods)\)/;

# now for the methods that only raise SAXExceptions
my @sax_methods = qw(
		     SAX2XMLReader::setProperty
		     SAX2XMLReader::setFeature
		     SAX2XMLReader::getProperty
		     SAX2XMLReader::getFeature
		    );
my $sax_methods = join '|', 
  map {s/::/_/; "_wrap_$_"} @sax_methods;

my $saxexception_regexp = qr/XS\(($sax_methods)\)/;


FILE: while(<FILE>) {

  substitute_line($_);

  if (/XS\(_wrap_XMLValidator_checkContent/) {
    fix_method_source(\*FILE,
		      \*TEMP,
		      'arg0->checkContent',
		      "            result = (int )arg0->checkContent(arg1,(QName **const)arg2,arg3);\n",
		      0
		     );
    next FILE;
  }

  # SWIG screws up the enumerations, so we make them ints in the 
  # header file and have to explicitly cast them by hand
  if (0) {

  if (/XS\(_wrap_DOM_DOMException_code_set/) {
    fix_method_source(\*FILE,
		      \*TEMP,
		      'arg0->code',
		      "    arg0->code = (DOM_DOMException::ExceptionCode)arg1;\n",
		      0
		     );
    next FILE;
  }

  if (/XS\(_wrap_IDOM_DOMException_code_set/) {
    fix_method_source(\*FILE,
		      \*TEMP,
		      'arg0->code',
		      "    arg0->code = (IDOM_DOMException::ExceptionCode)arg1;\n",
		      0
		     );
    next FILE;
  }

  if (/XS\(_wrap_DOM_RangeException_code_set/) {
    fix_method_source(\*FILE,
		      \*TEMP,
		      'arg0->code',
		      "    arg0->code = (DOM_RangeException::RangeExceptionCode)arg1;\n",
		      0
		     );
    next FILE;
  }

  if (/XS\(_wrap_IDOM_RangeException_code_set/) {
    fix_method_source(\*FILE,
		      \*TEMP,
		      'arg0->code',
		      "    arg0->code = (IDOM_RangeException::RangeExceptionCode)arg1;\n",
		      0
	      );
    next FILE;
  }
}

  # fix the PerlExceptionHandler::setExceptionHandler method
  if (/XS\(_wrap_PerlExceptionHandler_setExceptionHandler/) {
    print TEMP;

    skip_to_closing_brace(\*FILE);
    print TEMP <<"EOT";
    char *arg0 ;
    SV *arg1 ;
    SV *result;
    int argvi = 0;
    dXSARGS;
    
    if ((items < 2) || (items > 2)) 
    croak("Usage: PerlExceptionHandler_setExceptionHandler(class,function);");
    {
        arg1 = ST(1);
    }
    ST(argvi) = PerlExceptionHandler::setExceptionHandler(arg1);
    argvi++;
    XSRETURN(argvi);
}
EOT
    next FILE;
  }

  if (/XS\(_wrap_DOM_Attr_DOM_Attr__constructor__1/) {
    print TEMP;

    skip_to_closing_brace(\*FILE);
    print TEMP <<"EOT";
    DOM_Attr *arg0 ;
    int argvi = 0;
    DOM_Attr *result ;
    dXSARGS;
    
    if ((items < 1) || (items > 1)) 
    croak("Usage: DOM_Attr_DOM_Attr__constructor__1(self);");
    if (SWIG_ConvertPtr(ST(0),(void **) &arg0,SWIGTYPE_p_DOM_Attr) < 0) {
        croak("Type error in argument 1 of DOM_Attr_DOM_Attr__constructor__1. Expected %s", SWIGTYPE_p_DOM_Attr->name);
        XSRETURN(1);
    }
    result = new DOM_Attr((DOM_Attr const &)*arg0);
    ST(argvi) = sv_newmortal();
    SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_DOM_Attr);
    XSRETURN(argvi);
}
EOT
    next FILE;
  }

  # SAXParser::Parse has two possible exceptions
  if (/$saxexception_regexp/) {
    print TEMP;
  TEMP: while (<FILE>) {
      substitute_line($_);
      if (/catch \(const XMLException\& e\)/) {
	print TEMP <<"EOT";
    catch (const SAXNotSupportedException& e)
    {
            PerlExceptionHandler* handler;
            handler = PerlExceptionHandler::getInstance();
            handler->catchSAXNotSupportedException(e);
    }
    catch (const SAXNotRecognizedException& e)
    {
            PerlExceptionHandler* handler;
            handler = PerlExceptionHandler::getInstance();
            handler->catchSAXNotRecognizedException(e);
    }
EOT
	print TEMP;
	skip_to_closing_brace(\*FILE, \&substitute_line, \*TEMP);
	next FILE;
      } else {
	substitute_line($_);
	print TEMP;
	next TEMP;
      }
    }
  }

  # we need to move the new SWIG_TypeCheck() *after* the perl
  # header includes, because we now use sv_derived_from()
  if (!$PRINTED && /\#ifdef\s+PERL_OBJECT/) {
    $PRINTED = 1;
    print TEMP <<'EOT';
/* Check the typename */
SWIGRUNTIME(swig_type_info *) 
SWIG_TypeCheck(SV *sv, swig_type_info *ty)
{
  swig_type_info *s;
  if (!ty) return 0;        /* Void pointer */
  s = ty->next;             /* First element always just a name */
  while (s) {
    if (sv_derived_from(sv,s->name)) {
      if (s == ty->next) return s;
      /* Move s to the top of the linked list */
      s->prev->next = s->next;
      if (s->next) {
	s->next->prev = s->prev;
      }
      /* Insert s as second element in the list */
      s->next = ty->next;
      if (ty->next) ty->next->prev = s;
      ty->next = s;
      return s;
    }
    s = s->next;
  }
  return 0;
}

EOT
  }
  # now we substitute the line in SWIG_ConvertPTR()that 
  # calls SWIG_TypeCheck()
  if(/tc = SWIG_TypeCheck\(_c,_t\)/) {
    print TEMP<<"EOT";
    tc = SWIG_TypeCheck(sv,_t);
EOT
    next;
  }

  print TEMP;
}

close FILE;
close TEMP;

open(TEMP, "$temp_file")
  or die "Couldn't open $temp_file for reading";
open(FILE, ">$file")
  or die "Couldn't open $file for writing";

# put Perl in paragraph mode so that we read in entire blocks
# separated by blank lines
$/ = '';
$PRINTED = 0;
while(<TEMP>) {
  # we cut out the first occurrence of the SWIG_TypeCheck
  # and print everything else
  if (!$PRINTED && /SWIG_TypeCheck\(char \*c, swig_type_info \*ty\)/) {
    $PRINTED = 1;
    next;
  }
  print FILE;
}
close FILE;
close TEMP;

unlink $temp_file;

sub substitute_line {

  # we remove the RCS keyword from perl5.swg
  $_[0] = '' if $_[0] =~ /\$Header:/;

  # Help SWIG use stuff for later versions of Perl
  $_[0] =~ s/\b(sv_(undef|yes)|na)\b/PL_$1/g;

  # force SWIG to handle overloaded constructors
  $_[0] =~ s/(result\s+=)\s+.*arg0->(.*)__constructor__\d+/$1 new $2/;

}

# we always want to substitute every line so we default the argument
sub fix_method_source {
  fix_method(@_,\&substitute_line);
}
